% VES1dinv %
% 1D Inversion of Schlumberger Sounding Data %
% 
% Yunus Levent Ekinci and Alper Demirci, 2008. 
% A Damped Least-Squares Inversion Program for the Interpretation of 
%Schlumberger Sounding Curves. Journal of Applied Sciences, 8: 4070-4078.

close all; clear all;
clc;
format long;
[fname,fpath]=uigetfile('*.*','Select the input file');
fin=dlmread([fpath fname],'',1,0);

x = fin(:,1);          %AB/2;
ab2=x;
roa = fin(:,2);        %app resistivity

set(figure,'units','normalized','outerposition',[0 0 1 1]);
    
subplot(1,2,1),
loglog(x,roa,'k.');
axis([0.1 1000 1 1000])
xlabel('AB/2 (m)');
ylabel('\rho_a_p_p [\Omega m]');

[xa,ya]=ginput;

r=ya';
t=2.*(xa(1:size(xa,1)-1))';

m = [r t];
rinitial = r;
tinitial = t;
lr = length(r);
lt = length(t);
kr = 10e-10;
iteration = 1;
maxiteration = 500;
dfit = 1;

while iteration<maxiteration
r = m(1:lr);
t = m(1+lr:lr+lt);
   for i = 1:length(x)
    s = ab2(i);
   [g] = VES1dmod (r,t,s);
   roa1(i,:) = g;
   end
    
e1 = [log(roa)-log(roa1)];
dd = e1;
misfit1 = e1'*e1;
   if misfit1<kr
loglog(x,roa,'k.',x,roa1,'k');
axis([1 1000 10 1000])
xlabel('AB/2 (m)');
ylabel('Apparent Resistivity (Omega m)');
    break
   end

[A] = jacobian(ab2,x,r,t,lr,lt,roa,roa1);
[U S V] = svd(A,0);
ss = length(S);
say = 1;
k = 0;
   while say<ss
  diagS = diag(S);
  beta = S(say)*(dfit^(1/say));
     if beta<10e-5
     beta = 0.001*say;
     end

     for i4 = 1:ss
     SS(i4,i4) = S(i4,i4)/(S(i4,i4)^2+beta);
     end
     
  dmg = V*SS*U'*dd;
  mg = exp(log(m)+dmg');
  r = mg(1:lr);
  t = mg(1+lr:lr+lt);
  
     for i5 = 1:length(x)
     s = ab2(i5);
     [g] = VES1dmod (r,t,s);
     roa4(i5,:) = g;
     end

  e2 = [log(roa)-log(roa4)];
  misfit2 = e2'*e2;

    if misfit2>misfit1
     say = say+1;
      k = k+1;
       if k == ss-1
       iteration = maxiteration;
       say = ss+1;
       end

    else
    say = ss+1;
    m = mg;
    dfit = (misfit1-misfit2)/misfit1;
    iteration = iteration+1;
     a = iteration;
        if dfit<kr
        iteration = maxiteration;
        say = say+1;
        end

    end

   end

subplot(1,2,1),
loglog(x,roa,'k.',xa,ya,'ro',x,roa4,'k');
axis([0 1000 1 1000])
xlabel('AB/2 (m)');
ylabel('\rho_a_p_p [\Omega m]');
pause(0.001)
end

observed = roa;
calculated = roa4;
h = legend('obs','input','mod',1);
format bank;

rr = [0,r];
tt = [0,cumsum(t),max(t)*10];
subplot(1,2,2),
stairs(rr,tt,'k--');
rrr = [0,rinitial];
ttt = [0,cumsum(tinitial),max(t)*100];
xlim([0 10000]);
hold on;
subplot(1,2,2),
stairs(rrr,ttt,'r-');
set(gca,'Ydir','reverse');
set(gca,'Xscale','log');
legend('initial','final');
ylim([0 100]);
xlim([1 10000]);
xlabel('\rho [\Omega m]');
ylabel('Depth (m)');
rms = norm(roa4-roa) /sqrt(length(roa));


text(1000,10,['rms=' num2str(rms)]);
for ii=2:size(ttt,2)
text(2,60+(ii*5),['\rho_' num2str(ii-1) '=' num2str(rrr(ii)) ' \Omegam'])
text(50,60+(ii*5),['z=' num2str(ttt(ii-1)) 'm'])
end
